"use strict";

var Aes = require("./aes");
var PrivateKey = require("./key_private");
var PublicKey = require("./key_public");
var Signature = require("./signature");
var key_utils = require("./key_utils");
var config = require('./config');

var ecc = {
    Aes: Aes, PrivateKey: PrivateKey, PublicKey: PublicKey,
    Signature: Signature, key_utils: key_utils, config: config,

    /**
        TODO fast param (skips cpu entropy)
         @return {string} wif
    */
    randomPrivate: function randomPrivate() {
        return PrivateKey.randomKey().toString();
    },

    seedPrivate: function seedPrivate(seed) {
        return PrivateKey.fromSeed(seed).toString();
    },

    privateToPublic: function privateToPublic(wif) {
        return PrivateKey.fromWif(wif).toPublic().toString();
    },

    /** @return {boolean|string} true or error string */
    isValidPublic: function isValidPublic() {
        try {
            PublicKey.fromStringOrThrow.apply(PublicKey, arguments);
            return true;
        } catch (error) {
            return error;
        }
    },

    /** @return {boolean|string} true or error string */
    isValidPrivate: function isValidPrivate(wif) {
        try {
            PrivateKey.fromWif(wif);
            return true;
        } catch (error) {
            return error;
        }
    }
};

module.exports = ecc;